@@ -4,6 +4,8 @@ require 5.001;
 
 use strict;
 use warnings;
+use Carp;
+use parent 'LyricFinder::_Class';
 
 # LyricFinder - A Derived work, by (c) 2020 Jim Turner <turnerjw784 at yahoo.com> of:
 #
@@ -32,18 +34,9 @@ use warnings;
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-use vars qw(@ISA @EXPORT $VERSION);
-
-our $VERSION = '1.03';
-our $AGENT = "Mozilla/5.0 (X11; Linux x86_64; rv:80.0) Gecko/20100101 Firefox/80.0";
+our $VERSION = '1.10';
 our @FETCHERS = ();
-our $DEBUG = 0; # If you want debug messages, set debug to a true value, and
-				# messages will be output with warn.
-
-require Exporter;
-
-@ISA = qw(Exporter);
-@EXPORT = qw($AGENT $VERSION);
+our $DEBUG = 0;  # If you want debug messages, set debug to a true value
 
 my @supported_mods = (qw(ApiLyricsOvh AZLyrics Genius Letras Musixmatch Cache));
 
@@ -60,46 +53,16 @@ sub new
 {
 	my $class = shift;
 
-	my $self = {};
-	$self->{'agent'} =   $AGENT;
-	$self->{'cache'} =   '';
-	$self->{'Error'} =   'Ok';
-	$self->{'Source'} =  'none';
-	$self->{'Site'} =    '';
-	$self->{'Order'} =   '';
-	$self->{'Tried'} =   '';
-	$self->{'Url'} = '';
-	$self->{'Credits'} = [];
-
-	my %args = @_;
-	foreach my $i (keys %args) {
-		if ($i =~ s/^\-//) {
-			$self->{$i} = $args{"-$i"};
-		} else {
-			$self->{$i} = $args{$i};
-		}
-	}
-	$self->{'debug'} = $DEBUG  unless (defined $self->{'debug'});
+	my $self = $class->SUPER::new('', @_);
+	@{$self->{'_fetchers'}} = @FETCHERS;
+	unshift(@{$self->{'_fetchers'}}, 'Cache')  if ($haveit{'Cache'}
+			&& $self->{'-cache'} && $self->{'-cache'} !~ /^\>/);
 
 	bless $self, $class;   #BLESS IT!
 
 	return $self;
 }
 
-sub sources {
-	return wantarray ? @FETCHERS : \@FETCHERS;
-}
-
-sub source {
-	my $self = shift;
-	return $self->{'Source'};
-}
-
-sub url {
-	my $self = shift;
-	return $self->{'Url'};
-}
-
 sub order {
 	my $self = shift;
 	return wantarray ? split(/\,/, $self->{'Order'}) : $self->{'Order'};
@@ -110,45 +73,75 @@ sub tried {
 	return wantarray ? split(/\,/, $self->{'Tried'}) : $self->{'Tried'};
 }
 
-sub credits {
-	my $self = shift;
-	return wantarray ? @{$self->{'Credits'}} : join(', ', @{$self->{'Credits'}});
-}
+sub _fetch {
+	my ($self, $artist, $title, $fetchers) = @_;
 
-sub message {
-	my $self = shift;
-	return $self->{'Error'};
-}
-
-sub site {
-	my $self = shift;
-	return $self->{'Site'};
-}
-
-# Allow user to specify a different user-agent:
-sub agent {
-	my $self = shift;
-	if (defined $_[0]) {
-		$self->{'agent'} = $_[0];
-	} else {
-		return $self->{'agent'};
+	$self->_debug("LyricFinder::_fetch($artist, $title, $fetchers)!");
+	if (!$artist || !$title || ref $artist || ref $title) {
+		carp("e:_fetch() called without artist and title.");
+		return;
 	}
-}
 
-sub cache {
-	my $self = shift;
-	if (defined $_[0]) {
-		$self->{'cache'} = $_[0];
-	} else {
-		return $self->{'cache'};
+	if (!$fetchers || ref $fetchers ne 'ARRAY') {
+		carp("e:_fetch not given arrayref of fetchers to try");
+		return;
 	}
+
+	for my $fetcher (@$fetchers) {
+		$self->{'Url'} = '';
+		$self->_debug("..Trying fetcher $fetcher for artist:$artist title:$title");
+
+		my $fetcherpkg = __PACKAGE__ . "::$fetcher";
+		my $finderModule = 0;
+		eval "\$finderModule = new ${fetcherpkg}(\%{\$self});";
+		if ($@ || !$finderModule) {
+			carp("w:Failed to load sub-module $fetcherpkg ($@)");
+			next;
+		}
+		
+		# OK, we require()d this fetcher, try using it:
+		$self->{'Error'} = 'Ok';
+		$self->_debug("..Source module $fetcher loaded OK");
+		$self->{'Tried'} .= "$fetcher,";
+		if (!$finderModule->can('fetch')) {
+			$self->_debug("e:Source LyricFinder::$fetcher can't ->fetch($finderModule->{'Error'})");
+			next;
+		}
+	
+		$self->_debug("..Trying to fetch with $fetcher");
+		my $lyrics = $finderModule->fetch($artist, $title);
+		$self->{'Error'} = $finderModule->message();
+		$self->{'Url'} = $finderModule->url();
+		if ($self->{'Error'} eq 'Ok') {
+			$self->_debug("..Source: $fetcher returned lyrics");
+			if (defined($lyrics) && $lyrics =~ /\S/o) {
+				$self->{'Source'} = $finderModule->source();;
+				$self->{'Site'} = $finderModule->site();
+				@{$self->{'Credits'}} = $finderModule->credits();
+				$self->{'Tried'} =~ s/\,$//;
+				$self->_debug("i:Lyrics fetched from: ".$self->{'Source'});
+
+				return $lyrics;
+			}
+		}
+	}
+
+	# if we get here, we tried all sites we were asked to try, and none
+	# of them worked.
+	$self->{'Error'} = 'e:All sites failed to fetch lyrics!'  if ($#{$fetchers} > 0);
+	$self->{'Tried'} =~ s/\,$//;
+	
+	return undef;
 }
 
 sub fetch {
 	my ($self, $artist, $title, $fetcherspec, $limit) = @_;
 
-	my @tryfetchers;
+	my @tryfetchers = ();
+
+	$self->_debug("Mod::fetch($artist, $title, $fetcherspec)!");
 	$self->{'Tried'} = '';
+
 	$fetcherspec = 'random'  unless (defined($fetcherspec) && $fetcherspec);
 	$self->{'Source'} = 'none';
 	if ( $fetcherspec && !ref $fetcherspec && $fetcherspec !~ m'^auto$'i) {
@@ -167,13 +160,12 @@ sub fetch {
 					$usedcnt++;
 				}
 			}
-		} elsif ($fetcherspec =~ m'^Cache$'i) {
+		} elsif ($fetcherspec =~ m'^Cache$'i && $haveit{'Cache'}) {
 			@tryfetchers = ('Cache');
 		} elsif ($fetcherspec =~ m'^All$'i) {
 			push @tryfetchers, @FETCHERS;
 		} else { 
-			warn "$fetcherspec isn't a valid fetcher";
-			$self->{'Error'} = "s:Source (module) $fetcherspec isn't installed or is invalid!";
+			carp($self->{'Error'} = "s:Source (module) $fetcherspec isn't installed or is invalid!");
 			return;
 		}
 	} elsif (ref $fetcherspec eq 'ARRAY') {
@@ -182,7 +174,7 @@ sub fetch {
 			if (grep /$fetcher/, @FETCHERS) {
 				push @tryfetchers, $fetcher;
 			} else {
-				warn "$fetcher isn't a valid fetcher, ignoring";
+				carp("e:$fetcher isn't a valid fetcher, ignoring");
 			}
 		}
 	} else {  #really shouldn't end up here (since default=random now), but leaving in for now.
@@ -195,102 +187,9 @@ sub fetch {
 	return $self->_fetch($artist, $title, \@tryfetchers);
 }   # end of sub fetch.
 
-# actual implementation method - takes params $artist, $title, and an
-# arrayref of site modules to try.  Returns the result from the first
-# site that succeeded, or undef if all fail. ("fetchers" are site modules)
-sub _fetch {
-	my ($self, $artist, $title, $fetchers) = @_;
+1
 
-	if (!$artist || !$title || ref $artist || ref $title) {
-		warn "_fetch called incorrectly";
-		return;
-	}
-
-	if (!$fetchers || ref $fetchers ne 'ARRAY') {
-		warn "_fetch not given arrayref of fetchers to try";
-		return;
-	}
-
-fetcher:
-	for my $fetcher (@$fetchers) {
-		$self->{'Url'} = '';
-		$self->_debug("Trying fetcher $fetcher for artist:$artist title:$title");
-		print STDERR "----TRYING FETCHER=$fetcher= ----\n"  if ($self->{'debug'});
-
-		my $fetcherpkg = __PACKAGE__ . "::$fetcher";
-		my $finderModule = 0;
-		eval "\$finderModule = new ${fetcherpkg}(\%{\$self->{'$fetcher'}});";
-		if ($@ || !$finderModule) {
-			warn "Failed to load sub-module $fetcherpkg ($@)";
-			next fetcher;
-		}
-		
-		# OK, we require()d this fetcher, try using it:
-		$finderModule->agent($self->agent())  unless (defined $self->{$fetcher}->{'-agent'});
-		$finderModule->cache($self->cache())  unless (defined $self->{$fetcher}->{'-cache'});
-		$self->{'Error'} = 'Ok';
-		$self->_debug("Source module $fetcher loaded OK");
-		$self->{'Tried'} .= "$fetcher,";
-		if (!$finderModule->can('fetch')) {
-			$self->_debug("Source LyricFinder::$fetcher can't ->fetch($finderModule->{'Error'})");
-			next fetcher;
-		}
-	
-		$self->_debug("Trying to fetch with $fetcher");
-		my $f = $finderModule->fetch($artist, $title);
-		$self->{'Error'} = $finderModule->message();
-		$self->{'Url'} = $finderModule->url();
-		if ($self->{'Error'} eq 'Ok') {
-			$self->_debug("Source $fetcher returned lyrics");
-			if (defined($f) && $f) {
-				my $lyrics = _html2text($f);
-				if ($lyrics =~ /\S/o) {
-					$self->{'Source'} = $finderModule->source();;
-					$self->{'Site'} = $finderModule->site();
-					@{$self->{'Credits'}} = $finderModule->credits();
-					$self->{'Tried'} =~ s/\,$//;
-					print STDERR "-------FETCHED FROM=".$self->{'Source'}."=\n"  if ($self->{'debug'});
-
-					return $lyrics;
-				}
-			}
-		}
-		else {
-			next fetcher;
-		}
-	}
-
-	# if we get here, we tried all sites we were asked to try, and none
-	# of them worked.
-	$self->{'Error'} = 'e:All sites failed to fetch lyrics!'  if ($#{$fetchers} > 0);
-	$self->{'Tried'} =~ s/\,$//;
-	
-	return undef;
-}   # end of sub _fetch
-
-# nasty way to strip out HTML
-sub _html2text {
-	my $str = shift;
-	return ''  unless (defined $str);
-
-	$str =~ s/<br(.*?)>/\n/go;
-	$str =~ s/&gt;/>/go;
-	$str =~ s/&lt;/</go;
-	$str =~ s/&amp;/&/go;
-	$str =~ s/&quot;/\"/go;
-	$str =~ s/<.*?>//go;
-	return $str;
-}
-
-
-sub _debug {
-	my $self = shift;
-	my $msg = shift;
-	
-	warn $msg if $self->{'debug'};
-}
-
-1;
+__END__
 
 =head1 NAME
 
@@ -372,7 +271,7 @@ stored locally, called L<LyricFinder::Cache>.
 
 This module is derived from the (older) Lyrics::Fetcher collection of modules 
 by (c) 2007-2020 David Precious, but currently (as of December, 2020) supports 
-more lyric sites (4) and bundles all the supported site modules together here 
+more lyric sites (5) and bundles all the supported site modules together here 
 (simply install this one module).  We have reworked the "Cache" module to 
 cache lyrics files by artist and song title on disk in the user's desired 
 location.  LyricFinder is also truly object-oriented making interaction with 
@@ -509,6 +408,13 @@ Specifies whether debug information will be displayed (0: no, >0: yes).
 Default I<0> (no).  I<1> will display debug info.  There is currently only 
 one level of debug verbosity.
 
+=item B<-nostatus> => I<0 | 1> (I<false> or I<true>)
+
+If specified or given a I<true> value, sites may append some additional 
+information below the lyrics text.  Currently, only Musixmatch 
+(LyricFinder::Musixmatch) actually offers any additional information.  
+Default I<0> (false).
+
 =item B<site-module-name> => { I<"-option"> => I<value> [, I<"-option"> => I<value> ... ] }
 
 Specifies these same options for a specific site fetcher module.  These values 
@@ -581,7 +487,7 @@ the site with posting the lyrics on the site (if any) or an empty
 string, if none found.  NOTE:  The only site that supports this currently 
 is B<AZLyrics>.
 
-=item I<$string> = $finder->B<fetch>(I<$artist>, I<$title> [, I<$source> | I<\@sources>] [, I<$limit>])
+=item I<$string> = $finder->B<fetch>(I<$artist>, I<$title> [, I<$source> | I<\@sources> [, I<$limit>]])
 
 Attempt to fetch the lyrics for the given artist and title.
 A single source site module can be specified as a string ($source) or multiple 
@@ -605,7 +511,7 @@ If an array reference (a list) of modules are provided, they will be searched
 in the order they appear in the list.
 
 The currently-installed and supported modules are:  ApiLyricsOvh, AZLyrics, 
-Genius, and Musixmatch (NOTE the "x" in the spelling of "Musixmatch")!
+Genius, Letras, and Musixmatch (NOTE the "x" in the spelling of "Musixmatch")!
 
 Returns lyrics as a string (includes line-breaks appropriate for the user's 
 operating system), or an empty string, if no lyrics found.
@@ -662,7 +568,7 @@ The current version# of LyricFinder
 
 =head1 DEPENDENCIES
 
-L<HTML::Strip>, L<HTTP::Request>, L<LWP::UserAgent>
+L<HTML::Strip>, L<HTTP::Request>, L<LWP::UserAgent>, L<URI::Escape>
 
 =head1 BUGS
 
