@@ -1,132 +1,58 @@
 package LyricFinder::ApiLyricsOvh;
 
-use 5.008000;
 use strict;
 use warnings;
-use LWP::UserAgent;
-use HTTP::Request;
 use Carp;
-
-my $Source = 'ApiLyricsOvh';
-my $Site   = 'https://api.lyrics.ovh';
+use parent 'LyricFinder::_Class';
 
 our $haveLyricsCache;
 BEGIN {
+	$haveLyricsCache = 0;
 	eval "use LyricFinder::Cache; \$haveLyricsCache = 1; 1";
 }
 
-# the Default HTTP User-Agent we'll send:
-our $AGENT = "Mozilla/5.0 (X11; Linux x86_64; rv:80.0) Gecko/20100101 Firefox/80.0";
+my $Source = 'ApiLyricsOvh';
+my $Site   = 'https://api.lyrics.ovh';
+my $DEBUG  = 0;
 
 sub new
 {
 	my $class = shift;
 
-	my $self = {};
-	$self->{'agent'} = $AGENT;
-	$self->{'cache'} = '';
-	$self->{'Error'} = 'Ok';
-	$self->{'Source'} = $Source;
-	$self->{'Site'} = $Site;
-	$self->{'Url'} = '';
-	$self->{'Credits'} = [];
-
-	my %args = @_;
-	foreach my $i (keys %args) {
-		if ($i =~ s/^\-//) {
-			$self->{$i} = $args{"-$i"};
-		} else {
-			$self->{$i} = $args{$i};
-		}
-	}
-
+	my $self = $class->SUPER::new($Source, @_);
+	@{$self->{'_fetchers'}} = ($Source);
+	unshift(@{$self->{'_fetchers'}}, 'Cache')  if ($haveLyricsCache
+			&& $self->{'-cache'} && $self->{'-cache'} !~ /^\>/);
 	bless $self, $class;   #BLESS IT!
 
 	return $self;
 }
 
-sub source {
-	my $self = shift;
-	return $self->{'Source'};
-}
-
-sub url {
-	my $self = shift;
-	return $self->{'Url'};
-}
-
-sub order {
-	return wantarray ? ($Source) : $Source;
-}
-
-sub tried {
-	return order ();
-}
-
-sub credits {
-	my $self = shift;
-	return wantarray ? @{$self->{'Credits'}} : join(', ', @{$self->{'Credits'}});
-}
-
-sub message {
-	my $self = shift;
-	return $self->{'Error'};
-}
-
-sub site {
-	my $self = shift;
-	return $self->{'Site'};
-}
-
-# Allow user to specify a different user-agent:
-sub agent {
-	my $self = shift;
-
-	if (defined $_[0]) {
-		$self->{'agent'} = $_[0];
-	} else {
-		return $self->{'agent'};
-	}
-}
-
-sub cache {
-	my $self = shift;
-
-	if (defined $_[0]) {
-		$self->{'cache'} = $_[0];
-	} else {
-		return $self->{'cache'};
-	}
-}
-
 sub fetch {
 	my ($self, $artist_in, $song_in) = @_;
 
-	# reset the error var, change it if an error occurs.
-	$self->{'Error'} = 'Ok';
-	$self->{'Url'} = '';
+	$self->_debug("Letras::fetch($artist_in, $song_in)!");
 
-	unless ($artist_in && $song_in) {
-		carp($self->{'Error'} = "e:$Source.fetch() called without artist and song!");
-		return;
-	}
+	return ''  unless ($self->_check_inputs($artist_in, $song_in));
+	return ''  if ($self->{'Error'} ne 'Ok');
 
 	# first, see if we've got it cached:
-	if ($self->{'cache'} && $self->{'cache'} !~ /^\>/ && $haveLyricsCache) {
-		my $cache = new LyricFinder::Cache(-cache => $self->{'cache'});
+	$self->_debug("i:haveCache=$haveLyricsCache= -cachedir=".$self->{'-cache'}."=");
+	if ($haveLyricsCache && $self->{'-cache'} && $self->{'-cache'} !~ /^\>/) {
+		my $cache = new Mod::Cache(%{$self});
 		if ($cache) {
 			my $lyrics = $cache->fetch($artist_in, $song_in);
 			if (defined($lyrics) && $lyrics =~ /\w/) {
-				print STDERR "..Got lyrics from cache.\n"  if ($self->{'-debug'});
+				$self->_debug("..Got lyrics from cache.");
 				$self->{'Source'} = 'Cache';
 				$self->{'Site'} = $cache->site();
 				$self->{'Url'} = $cache->url();
+
 				return $lyrics;
 			}
 		}
 	}
 
-	$self->{'Source'} = $Source;
 	$self->{'Site'} = $Site;
 
 	(my $artist = $artist_in) =~ s#\s*\/.*$##;    #ONLY USE 1ST ARTIST, IF MORE THAN ONE!
@@ -138,61 +64,24 @@ sub fetch {
 
 	# Their URLs look like e.g.:
 	# https://api.lyrics.ovh/v1/Dire%20straits/heavy%sfuel%s
-	$self->{'Url'} = "https://api.lyrics.ovh/v1/${artist}/$song";
-	my $ua = LWP::UserAgent->new(
-		ssl_opts => { verify_hostname => 0, },
-	);
-	$ua->timeout(10);
-	$ua->agent($self->{'agent'});
-	$ua->protocols_allowed(['https']);
-	$ua->cookie_jar( {} );
-	push @{ $ua->requests_redirectable }, 'GET';
-	(my $referer = $self->{'Url'}) =~ s{^(\w+)\:\/\/}{};
-	my $protocol = $1;
-	$referer =~ s{\/.+$}{\/};
-	my $host = $referer;
-	$host =~ s{\/$}{};
-	$referer = $protocol . '://' . $referer;
-	my $req = new HTTP::Request 'GET' => $self->{'Url'};
-	$req->header(
-		'Accept' =>
-			'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
-		'Accept-Language'           => 'en-US,en;q=0.5',
-		'Accept-Encoding'           => 'gzip, deflate',
-		'Connection'                => 'keep-alive',
-		'Upgrade-insecure-requests' => 1,
-		'Host'                      => $host,
-	);
-
-	my $res = $ua->request($req);
-
-	if ($res->is_success) {
-		my $lyrics = $self->_parse($res->decoded_content);
-		if ($self->{'cache'} && $self->{'cache'} !~ /^\</ && $lyrics && $haveLyricsCache) {
-			# cache the fetched lyrics, if we can:
-			my $cache = new LyricFinder::Cache(-cache => $self->{'cache'});
-			$cache->save($artist_in, $song_in, $lyrics)  if ($cache);
-		}
-		return $lyrics;
-	} else {
-		if ($res->status_line =~ /^404/) {
-			$self->{'Error'} = "..$Source - Lyrics not found.";
-		} else {
-			carp($self->{'Error'} = "e:$Source - Failed to retrieve ".$self->{'Url'}
-					.' ('.$res->status_line.').');
-		}
-		return;
+	$self->{'Url'} = "${Site}/v1/${artist}/$song";
+	my $lyrics = $self->_web_fetch($artist_in, $song_in);
+	if ($lyrics && $haveLyricsCache && $self->{'-cache'} && $self->{'-cache'} !~ /^\</) {
+		$self->_debug("=== WILL CACHE LYRICS! ===");
+		# cache the fetched lyrics, if we can:
+		my $cache = new LyricFinder::Cache(%{$self});
+		my $res = 0;
+		$cache->save($artist_in, $song_in, $lyrics)  if ($cache);
 	}
+	return $lyrics;
 }
 
-# Internal use only functions:
-
 sub _parse {
 	my $self = shift;
 	my $html = shift;
 
-	if (my ($goodbit) = $html =~
-		m{\{\"lyrics\"\:\"([^\"]+)\"}msi)
+	$self->_debug("ApiLyricsOvh::_parse()!");
+	if (my ($goodbit) = $html =~ m{\{\"lyrics\"\:\"([^\"]+)\"}msi)
 	{
 		my $text = '';
 		# convert literal "\" followed by "r" or "n", etc. to "\r" or "\n" characters respectively:
@@ -200,28 +89,15 @@ sub _parse {
 
 		# fix apparent site bug where they use "\n\n" where they appear to mean "\r\n" (excess double-lines):
 		$text =~ s/\n\n/\n/gs;
-		# normalize Windowsey \r\n sequences:
-		$text =~ s/\r+//gs;
-		# strip off pre & post padding with spaces:
-		$text =~ s/^ +//mg;
-		$text =~ s/ +$//mg;
-		# clear up repeated blank lines:
-		$text =~ s/(\R){2,}/\n\n/gs;
-		# and remove any blank top lines:
-		$text =~ s/^\R+//s;
-		$text =~ s/\R\R+$/\n/s;
-		$text .= "\n"  unless ($text =~ /\n$/s);
-		# now fix up for either Windows or Linux/Unix:
-		$text =~ s/\R/\r\n/gs  if ($^O =~ /Win/);
 
-		return $text;
+		return $self->_normalize_lyric_text($self->_html2text($text));
 	} else {
 		carp($self->{'Error'} = "e:$Source - Failed to identify lyrics on result page.");
 		return '';
 	}
-} # end of sub parse
+}
 
-1;
+1
 
 __END__
 
